import React, { useState, useRef } from "react";
import { invoke } from "@tauri-apps/api/core";
import { open } from "@tauri-apps/plugin-dialog";
import "./style.css";

// Simple debounce hook
function useDebouncedCallback(callback, delay) {
  const timeoutRef = useRef(null);
  return (...args) => {
    if (timeoutRef.current) clearTimeout(timeoutRef.current);
    timeoutRef.current = setTimeout(() => {
      callback(...args);
    }, delay);
  };
}

export default function App() {
  const [result, setResult] = useState("");
  const [loading, setLoading] = useState(false);
  const [inputPath, setInputPath] = useState("");
  const [format, setFormat] = useState("mp4");
  const [videoCodec, setVideoCodec] = useState("x264");
  const [preset, setPreset] = useState("veryslow");

  const handlePickFile = async () => {
    const selected = await open({
      filters: [
        { name: "Videos", extensions: ["mp4", "mkv", "webm", "mov", "avi"] },
      ],
    });
    if (typeof selected === "string") {
      setInputPath(selected);
    }
  };

  const handleReencode = async () => {
    if (!inputPath) {
      setResult("Please select a video file.");
      return;
    }
    setLoading(true);
    setResult("Re-encoding...");
    try {
      invoke("reencode_video", {
        inputPath,
        outputFormat: format,
        videoCodec,
        preset,
      }).then(setResult);
    } catch (e) {
      setResult("Error: " + e);
    } finally {
      setLoading(false);
    }
  };

  const debouncedReencode = useDebouncedCallback(handleReencode, 1000);

  return (
    <div className="card">
      <button type="button" onClick={handlePickFile} disabled={loading}>
        Select Video
      </button>
      <span style={{ marginLeft: 8 }}>{inputPath ? inputPath : "No file selected"}</span>
      <div style={{ margin: "12px 0" }}>
        <label htmlFor="format-select">Output format: </label>
        <select
          id="format-select"
          value={format}
          onChange={e => setFormat(e.target.value)}
          disabled={loading}
        >
          <option value="mp4">mp4</option>
          <option value="mkv">mkv</option>
          <option value="webm">webm</option>
          <option value="mov">mov</option>
          <option value="avi">avi</option>
        </select>
      </div>
      <div style={{ margin: "12px 0" }}>
        <label htmlFor="codec-select">Video codec: </label>
        <select
          id="codec-select"
          value={videoCodec}
          onChange={e => setVideoCodec(e.target.value)}
          disabled={loading}
        >
          <option value="x264">x264</option>
          <option value="x265">x265</option>
        </select>
      </div>
      <div style={{ margin: "12px 0" }}>
        <label htmlFor="preset-select">Preset: </label>
        <select
          id="preset-select"
          value={preset}
          onChange={e => setPreset(e.target.value)}
          disabled={loading}
        >
          <option value="veryslow">veryslow</option>
          <option value="slower">slower</option>
          <option value="slow">slow</option>
          <option value="medium">medium</option>
          <option value="fast">fast</option>
          <option value="faster">faster</option>
          <option value="ultrafast">ultrafast</option>
        </select>
      </div>
      <button
        id="reencode-btn"
        type="button"
        onClick={debouncedReencode}
        disabled={loading || !inputPath}
      >
        Re-encode Video
      </button>
      {loading && <div className="spinner" aria-label="Encoding in progress" />}
      <p id="ffmpeg-result">{result}</p>
    </div>
  );
}
