use std::path::Path;
use std::process::Command;
use tauri::command;

#[command]
fn reencode_video(
    input_path: String,
    output_format: String,
    video_codec: Option<String>,
    preset: Option<String>,
) -> Result<String, String> {
    // Determine output path
    let input = Path::new(&input_path);
    let stem = input
        .file_stem()
        .and_then(|s| s.to_str())
        .ok_or("Invalid input file name")?;
    let parent = input.parent().unwrap_or_else(|| Path::new("."));
    let output_path = parent.join(format!("{}_reencoded.{}", stem, output_format));

    // Set codec and preset, with defaults
    let codec = video_codec.unwrap_or_else(|| "x264".to_string());
    let preset = preset.unwrap_or_else(|| "veryslow".to_string());
    let codec_flag = match codec.as_str() {
        "x264" => "libx264",
        "x265" => "libx265",
        other => other, // fallback, allow custom
    };

    // Build ffmpeg command
    let status = Command::new("ffmpeg")
        .arg("-y")
        .arg("-i")
        .arg(&input_path)
        .arg("-c:v")
        .arg(codec_flag)
        .arg("-preset")
        .arg(&preset)
        .arg(output_path.to_str().unwrap())
        .status()
        .map_err(|e| format!("Failed to start ffmpeg: {}", e))?;

    if status.success() {
        Ok(format!(
            "Re-encoded video saved to: {}",
            output_path.display()
        ))
    } else {
        Err("ffmpeg failed to re-encode the video".to_string())
    }
}

#[cfg_attr(mobile, tauri::mobile_entry_point)]
pub fn run() {
    tauri::Builder::default()
        .plugin(tauri_plugin_dialog::init())
        .invoke_handler(tauri::generate_handler![reencode_video])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
